
package documenttools;

import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyledDocument;

/**
 * An instance of the class is used to simplify the creation of a
 * styled document.
 * @author Projekte
 */
public class StyledDocumentCreator {
    /**
     * A reference to the document to be created.
     */
    private StyledDocument document;
    /**
     * The current style. The current style is used for all insert
     * operations as long as no other current style is defined.
     */
    private Style currentStyle = null;
    /**
     * A permanent reference to the platform-specific newline
     * sequence.
     */
    final private String NL = System.getProperty("line.separator");
    
    /**
     * Create an instance of StyledDocumentCreator.
     * @param doc An empty or non-empty styled document, completely
     * prepared with all document styles. 
     */
    public StyledDocumentCreator(StyledDocument doc) {
        document = doc;
    }
    
    /**
     * Access the document
     * @return the document
     */
    public StyledDocument getDocument() {
        return document;
    }
    
    /**
     * Append a string to the end of the document. The string is appended
     * with the current presentation style.
     * @param string
     * @return The method returns the instance to facilitate message chaining.
     */
    public StyledDocumentCreator append (String string) {
        try {
            document.insertString(document.getLength(), string, currentStyle);
        }
        catch (BadLocationException ex)
        {

        };
        return this;
    }
        
    public StyledDocumentCreator append(int integerValue) {
        try {
            document.insertString(document.getLength(), 
                    Integer.toString(integerValue),
                    currentStyle);
        }
        catch (BadLocationException ex)
        {

        };
        return this;
    }
    
    public StyledDocumentCreator append(double integerValue) {
        try {
            document.insertString(document.getLength(), 
                    Double.toString(integerValue),
                    currentStyle);
        }
        catch (BadLocationException ex)
        {

        };
        return this;
    }
    /**
     * Append a newline to the end of the document.
     * @return The method returns the instance to facilitate message chaining.
     */
    public StyledDocumentCreator nl() {
        try {
            document.insertString(document.getLength(),
                    NL,
                    currentStyle);
        }
        catch (BadLocationException ex)
        {

        };
        return this;
    }
    
    public StyledDocumentCreator nl(int cnt) {
        try {
            for (int i = 0; i < cnt; i++)
            {
                document.insertString(document.getLength(),
                        NL,
                        currentStyle);
            }
        }
        catch (BadLocationException ex)
        {

        };        
        return this;
    }
    /**
     * Store the style named 'styleName' as the new current presentation style.
     * The style is searched in the document.
     * @param styleName The name of the style to be used as the new current presentation style.
     * @return The method returns the instance to facilitate message chaining.
     */
    public StyledDocumentCreator setStyle(String  styleName) {
        currentStyle = document.getStyle(styleName);
        return this;
    }
}
