
package notions;

/**
 *
 * @author B. Gaertner
 */
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;

/**
 * Eine Instanz stellt die beiden Seiten einer chemischen Gleichungg
 * dar.
 * Bei der Herstellung der Instanz wird nicht geprft, ob die Stoffmenge
 * und die Ladunge beider Seiten bereinstimmen. Diese Prfung sollte am besten
 * vor dem Aufruf des Konstruktors geschehen. Im Augenblick geschieht es
 * aber erst nach der Herstellung einer Instanz und auch nur, denn die
 * berprfung ausdrcklich verlangt wird.
 * @author Projekte
 */
public class Reaktionsgleichung {

    LinkedList<Stoffmenge> lhs;
    LinkedList<Stoffmenge> rhs;

    public Reaktionsgleichung (LinkedList<Stoffmenge> arg1,
            LinkedList<Stoffmenge> arg2) {
        lhs = arg1;
        rhs = arg2;
    }

    /**
     * Sollte so umgendert werden, dass die Herstellung einer
     * aussagekrftigen Fehlermeldung mglich wird.
     * @return
     */
    public boolean istWohlgeformt() {
        HashMap<String, Integer> elementsOnTheLeft;
        HashMap<String, Integer> elementsOnTheRight;
        elementsOnTheLeft = new HashMap<String, Integer>();
        elementsOnTheRight = new HashMap<String, Integer>();
        this.berechneStoffmengen(lhs, elementsOnTheLeft);
        this.berechneStoffmengen(rhs, elementsOnTheRight);
        return elementsOnTheLeft.equals(elementsOnTheRight);
    }

        public AnalyseErgebnisGleichung istWohlgeformtXX() {
        HashMap<String, Integer> elementsOnTheLeft;
        HashMap<String, Integer> elementsOnTheRight;
        elementsOnTheLeft = new HashMap<String, Integer>();
        elementsOnTheRight = new HashMap<String, Integer>();
        this.berechneStoffmengen(lhs, elementsOnTheLeft);
        this.berechneStoffmengen(rhs, elementsOnTheRight);
        return new AnalyseErgebnisGleichung( elementsOnTheLeft, elementsOnTheRight);
    }


    public LinkedList<Stoffmasse> berechneMassen() {
        LinkedList<Stoffmasse> masses;
        masses = new LinkedList<Stoffmasse>();
        this.berechneMassen(lhs, masses);
        this.berechneMassen(rhs, masses);
        return masses;
    }

    private void berechneMassen(LinkedList<Stoffmenge> list, LinkedList<Stoffmasse> masses) {
        Iterator<Stoffmenge> iterator = list.listIterator();
        while (iterator.hasNext()) {
            Stoffmenge ar = iterator.next();
            Partikel mp = ar.substanz;
            float m = mp.berechneMassen(ar.menge);
            masses.addLast(new Stoffmasse(mp, ar.menge, m));

        }
    }

    public void toHtml(StringBuilder sb) {
        this.enumerationToHtml(lhs, sb);
        sb.append(" &rarr; ");
        this.enumerationToHtml(rhs, sb);
    }

    private void enumerationToHtml(LinkedList<Stoffmenge> list, StringBuilder sb) {
        Iterator<Stoffmenge> iterator = list.listIterator();
        while (iterator.hasNext()) {
            Stoffmenge ar = iterator.next();
            ar.toHtml(sb);
            if (iterator.hasNext()) {
                sb.append(" + ");
            }
        }
    }

    private void berechneStoffmengen(LinkedList<Stoffmenge> c, HashMap<String, Integer> d) {
        Iterator<Stoffmenge> iterator = c.listIterator();
        while (iterator.hasNext()) {
            Stoffmenge ar = iterator.next();
            ar.berecheStoffmenge(d);
        }
    }

}