
package notions;


import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map.Entry;

/**
 * german notion: Summenformel.  (molecularFormula)
 * Instances are used to compute molar masses
 * @author Projekte
 *
 */
public class Verbindung extends Stoff {

    LinkedList<Teilformel> konstituenten;

    public Verbindung() {
        konstituenten = new LinkedList<Teilformel> ();
    }

    public void add(Teilformel fc) {
        konstituenten.addLast(fc);
    }

    @Override
    public float molekularmasse() {
        float sum = 0.0f;
        Iterator<Teilformel> iterator = konstituenten.iterator();
        while (iterator.hasNext()) {
                Teilformel fc = iterator.next();
                sum += fc.struktur.molekularmasse()*fc.anzahl;
        }
        return sum;
    }

    @Override
    public boolean istVerbindung() {
        return true;
    }

    @Override
    public void toHtml(StringBuilder sb) {
        Iterator<Teilformel> iterator = konstituenten.iterator();
        while (iterator.hasNext()) {
                Teilformel fc = iterator.next();
                fc.toHtml(sb);
        }
    }

    @Override
    public float berechneMassen(int f) {
        return f*this.molekularmasse();
    }

    @Override
    public void berechneStoffmengen(int factor, HashMap<String, Integer> d) {
        Iterator<Teilformel> iterator = konstituenten.iterator();
        while (iterator.hasNext()) {
                Teilformel fc = iterator.next();
                fc.struktur.berechneStoffmengen(factor*fc.anzahl, d);
        }
    }

    /**
     * http://de.wikipedia.org/wiki/Hill-System
     * @return
     */
    public String condensedHillFormula() {
        HashMap<String, Integer> d = new HashMap<String, Integer>();
        this.berechneStoffmengen(1, d);
        StringBuilder builder = new StringBuilder();
        if (d.containsKey("C")) {
            builder.append("C");
            if (d.get("C") > 1 ) {
                builder.append(Integer.toString(d.get("C")));
            }
            d.remove("C");
            if (d.containsKey("H")) {
                builder.append("H");
                if (d.get("H") > 1 ) {
                    builder.append(Integer.toString(d.get("H")));
                }
                d.remove("H");
            }
        }
        String[] symbols = new String[d.size()];
        Iterator<Entry<String, Integer>> iterator = d.entrySet().iterator();
        int i = 0;
        while (iterator.hasNext()) {
            symbols[i++] = iterator.next().getKey();
        }
        Arrays.sort(symbols);
        for (int j = 0; j < symbols.length; j++) {
            builder.append(symbols[j]);
            if (d.get(symbols[j]) > 1) {
                builder.append(Integer.toString(d.get(symbols[j])));
            }
        }
        return builder.toString();
    }

    /**
     * diese Variante ermglicht spter die Herstellung einer einfachen
     * Zeichenreihe und die Herstellung einer HTML-codierten Darstellung.
     * @param pse
     * @return
     */
    public Verbindung hillOrdnung(Periodensystem pse) {
        HashMap<String, Integer> d = new HashMap<String, Integer>();
        this.berechneStoffmengen(1, d);
        Verbindung result = new Verbindung();
        if (d.containsKey("C")) {
            result.add(new Teilformel(pse.elementMitName("C"), d.get("C")));
            d.remove("C");
            if (d.containsKey("H")) {
                result.add(new Teilformel(pse.elementMitName("H"), d.get("H")));
                d.remove("H");
            }
        }
        String[] symbols = new String[d.size()];
        Iterator<Entry<String, Integer>> iterator = d.entrySet().iterator();
        int i = 0;
        while (iterator.hasNext()) {
            symbols[i++] = iterator.next().getKey();
        }
        Arrays.sort(symbols);
        for (int j = 0; j < symbols.length; j++) {
            result.add(new Teilformel(pse.elementMitName(symbols[j]), d.get(symbols[j])));
        }
        return result;
    }
}
