
package parser;

import notions.*;
import java.util.LinkedList;

/**
 * Aufgaben dieser Klasse:
 *
 */
public class FormulaParser {
    Periodensystem pse;
    TokenStream reader;
    String[] specialSymbols;

    public FormulaParser (Periodensystem p) {
        specialSymbols = new String[4];
        specialSymbols[0] = "->";
        specialSymbols[1] = "+";
        specialSymbols[2] = "(";
        specialSymbols[3] = ")";
        pse = p;
    }

    public boolean atEnd() {
        return reader.atEnd();
    }

    private AbstractToken next()  {
        return reader.next();
    }

    public Reaktionsgleichung parseFormula(String stringToParse)
             throws NotificationOfSyntaxError {
        LinkedList<Stoffmenge> lhs;
        LinkedList<Stoffmenge> rhs;

        reader = new TokenStream(new StringTokenizer(stringToParse, specialSymbols));

        lhs = this.enumerationOfSubstances();
        AbstractToken currentToken = reader.peek();
        boolean cont = this.isSymbol(currentToken, "->");
        if (cont) {
            reader.next();
        } else if (currentToken.isUnclassifiedToken()) {
            this.reportError(SyntaxError.UNEXPECTED_TOKEN);
        } else if (currentToken.isEndToken()) {
            this.reportError(SyntaxError.END_OF_LINE);
        }
        rhs = this.enumerationOfSubstances();
        return new Reaktionsgleichung(lhs, rhs);
    }

    /**
     *
     * @param stringToParse
     * @return
     * @throws NotificationOfSyntaxError
     */
    public Verbindung parseComposite(String stringToParse)
             throws NotificationOfSyntaxError {
        reader = new TokenStream(new StringTokenizer(stringToParse, specialSymbols));
        Verbindung v = this.parseComposite();
        if (!reader.peek().isEndToken()) {
            this.reportError(SyntaxError.UNEXPECTED_TOKEN, reader.peek());
        }
        return v;
    }

    /**
     * lies eine Aufzhlung von Substanzformeln, die durch Pluszeichen
     * miteinander verbunden sind. Es muss mindestens eine Stoff genannt sein.
     * @return
     * @throws NotificationOfSyntaxError
     */
    private LinkedList<Stoffmenge> enumerationOfSubstances()
              throws NotificationOfSyntaxError {
        AbstractToken currentToken;
        LinkedList<Stoffmenge> list;
        list = new LinkedList<>();
        int amount;
        boolean cont;
        //while (! this.atEnd() && cont) {
        do {
           if (reader.atEnd()) {
               this.reportError(SyntaxError.PRIMARY_EXPECTED);
           }
           currentToken = reader.peek();
           if (currentToken.isNumber()) {
               this.next();
               amount = Integer.parseInt(currentToken.getString());
           } else {
               amount = 1;
           }
           Partikel c = this.parseParticle(); // compound substance or charge
           list.addLast(new Stoffmenge(c, amount));

           /*
           currentToken = reader.peek();
           cont = this.isSymbol(currentToken, "+");
           if (cont) {
               reader.next();
           }  */
           cont = reader.peekFor("+");
        } while (cont);
        return list;
    }

    private Partikel parseParticle() throws NotificationOfSyntaxError {
        //  for electrocemical rections: detect charges here
        return this.parseComposite();
    }
    /**
     *
     * @return
     * @throws NotificationOfSyntaxError
     */
    private Verbindung parseComposite() throws NotificationOfSyntaxError {
        AbstractToken currentToken;
        currentToken = reader.peek();
        Verbindung mf;
        Stoff substance;
        mf = new  Verbindung();
        //while (!this.atEnd() && (currentToken.isName() || this.isSymbol(currentToken, "("))) {
        do {
            if (currentToken.isEndToken()) {
                this.reportError(SyntaxError.PRIMARY_EXPECTED);
            }
            Token t2 = (Token) currentToken;
            if (t2.isSymbol("(")) {
                reader.next();
                substance = this.parseComposite();
                if (this.atEnd()) {
                    this.reportError(SyntaxError.END_OF_LINE);
                } else {
                    currentToken = reader.next();
                    if (currentToken.isEndToken()) {
                        this.reportError(SyntaxError.RIGHT_PARENTHESIS_EXPECTED);
                    } else {
                        this.passOverSymbol((Token)currentToken, ")");
                    }
                }
            } else {
                substance = this.readElement();
            }
            int amount = 1;
            if(reader.peek().isNumber()) {
                NumberToken n = (NumberToken) reader.next();
                amount = Integer.parseInt(n.getString());
            }
            mf.add(new Teilformel(substance, amount));
            currentToken = reader.peek();
        }
        while (!this.atEnd() && (currentToken.isName() || this.isSymbol(currentToken, "(")));
        return mf;
    }

    private Stoff readElement() throws NotificationOfSyntaxError {
        AbstractToken token = reader.next();
        if (token.isEndToken()) {
            this.reportError(SyntaxError.END_OF_LINE);
        } else {
            Token t = (Token) token;
            String name = t.getString();
            Element element = pse.elementMitName(name);
            if(element == null) {
                this.reportError(SyntaxError.PRIMARY_EXPECTED, t);
            }
            return element;
        }
        return null;
    }

    private boolean isSymbol (AbstractToken t, String s) {
        if (t.isEndToken()) {
            return false;
        } else {
            Token t2 = (Token) t;
            return t2.isSymbol(s);
        }
    }

    private void passOverSymbol(Token t, String s)
              throws NotificationOfSyntaxError {
        if (!t.isSymbol(s)) {
            this.reportError(SyntaxError.RIGHT_PARENTHESIS_EXPECTED);
        }
    }

    private void reportError(SyntaxError s) throws NotificationOfSyntaxError {
        AbstractToken t = reader.peek();
        throw new NotificationOfSyntaxError(t.getStartPosition(), t.getEndPosition(), s);
    }

    private void reportError(SyntaxError s, AbstractToken t) throws NotificationOfSyntaxError {
        throw new NotificationOfSyntaxError(t.getStartPosition(), t.getEndPosition(), s);
    }
}
