
package parser;

import java.util.Set;

/**
 *
 * @author B. Gaertner
 */
public class StringTokenizer {

    private String stringToParse;
    private String[] specialStrings;
    int currentPos;

    public StringTokenizer(String s, String[] ss) {
        stringToParse = s;
        currentPos = 0;
        specialStrings = ss;
    }

    public boolean atEnd() {
        return currentPos >= stringToParse.length();
    }

    public Token nextToken()
    {Token t;
        t = this.advanceOver(specialStrings);
        if (t != null) {

            return t;
        }
        return this.readDenotation();
    }

    public char nextChar() {
        return stringToParse.charAt(currentPos++);
    }

    public char peek() {
        return stringToParse.charAt(currentPos);
    }

    public void skipWhitespace() {
        while (!this.atEnd() && this.currentIsWhitespace()) {
            currentPos += 1;
        }
    }

    private boolean currentIsWhitespace() {
        if (currentPos >= stringToParse.length()) {
            return false;
        }
        char ch = stringToParse.charAt(currentPos);
        /*
        if (ch == ' ' || ch == '\n' || ch == '\t') {
            return true;
        }
        return false; */
        return Character.isWhitespace(ch);
    }

    public void skipOverCharsIn(Set<Character> s) {
        while(!this.atEnd() && s.contains(this.peek())) {
            currentPos += 1;
        }
    }

    public Token advanceOver(String[] s) {
        /*
        if (!this.atEnd() && s.contains(this.peek())) {
            int pos = currentPos;
            char[] chars = new char[1];
            chars[0] = this.nextChar();
            String st = new String(chars);
            return new Token(st, pos);
        } else {
            return null;
        } */
        if (this.atEnd()) {
            return null;
        }
        for (int i = 0; i < s.length; i++) {
            String givenString = s[i];
            if (! (currentPos + givenString.length() >= stringToParse.length())) {
                int endPos = currentPos + givenString.length();
                String nextChars = stringToParse.substring(currentPos, endPos);
                if (nextChars.equals(givenString)) {
                    int startPos = currentPos;
                    currentPos += givenString.length();
                    return new Token (nextChars, startPos);
                }
            }
        }
        return null;
    }

    public Token readDenotation() {
        if (this.atEnd()) {
            return null;
        } else {
            char ch = this.peek();
            if (this.isDigit(ch)) {
                return this.readNumber();
            } else {
                if (this.isCapitalCharacter(ch)) {
                    return this.readName();
                } else {
                    int startPos;
                    startPos = currentPos;
                    while (!this.atEnd() && !Character.isWhitespace(this.peek())) {
                        currentPos += 1;
                    }
                    return new UnclassifiedToken(
                            stringToParse.substring(startPos, currentPos),
                            startPos);
                }
            }
        }
    }

    /**
     * Diese Methode identifziert nur ganze Zahlen
     * @return
     */
    public Token readNumber() {
        String st;
        int startPos;
        startPos = currentPos;
        while (!this.atEnd() && this.isDigit(this.peek())) {
            currentPos += 1;
        }
        st = stringToParse.substring(startPos, currentPos);
        return new NumberToken(st, startPos);
    }

    /**
     * Das Symbol eines chemischen Elements beginnt mit einem Grobuchstaben,
     * dem Kleinbuchstaben folgen knnen.
     * @return
     */
    public Token readName() {
        String st;
        int startPos;
        startPos = currentPos;
        currentPos += 1;
        while (!this.atEnd() && this.isSmallCharacter(this.peek())) {
            currentPos += 1;
        }
        st = stringToParse.substring(startPos, currentPos);
        return new NameToken(st, startPos);
    }

    private boolean advanceOverChar(char charToSkip) {
        if (!this.atEnd() && (this.peek()) == charToSkip) {
            currentPos += 1;
            return true;
        } else {
            return false;
        }
    }

    private boolean isDigit(char ch) {
        return ch >= '0' && ch <= '9';
    }

    private boolean isCharacter(char ch) {
        return (ch >= 'A' && ch <= 'Y') || (ch >= 'a' && ch <= 'z');
    }

    private boolean isCapitalCharacter(char ch) {
        return (ch >= 'A' && ch <= 'Y');
    }

    private boolean isSmallCharacter(char ch) {
        return (ch >= 'a' && ch <= 'z');
    }

    private boolean isAlphaNumeric(char ch) {
        return this.isCharacter(ch) || this.isDigit(ch);
    }
}
