
package window;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.DefaultButtonModel;
import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextPane;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;
import notions.AnalyseErgebnisGleichung;
import notions.Reaktionsgleichung;
import notions.Stoffmasse;
import parser.FormulaParser;
import parser.NotificationOfSyntaxError;

/**
 * Dieser Rechner berechnet fr eine einzugebende Reaktionsgleichung die
 * Massen der Reaktionsprodukte auf der Grundlage der in der Reaktionsgleichung
 * angegebenen Stoffmengen.
 * @author B. Gaertner
 */
public class CalculatorWindow extends JInternalFrame
        implements ActionListener,
                   InternalFrameListener,
                   LayoutManager {

    private ApplicationModel model;
    private PlainDocument document;
    private DefaultTableModel mengentabelle;
    private JTextArea expressionEditArea;
    private JTextPane resultField;
    private JTable tabelle;
    private JButton compileButton;

    public CalculatorWindow(ApplicationModel m, Dimension d) {
        super("Rechner",
              true, //resizable
              true, //closable
              true, //maximizable
              true);//iconifiable
        model = m;
        document = new PlainDocument();
        mengentabelle = new DefaultTableModel(0, 3);
        this.createGUI(d);
    }

    private void createGUI(Dimension d) {
        setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
        this.addInternalFrameListener(this);
        addComponentsToPane(this.getContentPane());
        setSize(d.width*5/10, d.height*5/10);
        setLocation(0, 10);
    }

    private void addComponentsToPane(Container container) {
        container.setLayout(this);
        Font labelFont = new Font(Font.SERIF, Font.BOLD, 16);
        JLabel titel1 = new JLabel("Reaktionsgleichung");
        titel1.setFont(labelFont);

        expressionEditArea = new JTextArea(document);
        Font fnt = new Font(Font.MONOSPACED, Font.PLAIN, 18);
        expressionEditArea.setFont(fnt);

        JScrollPane scrollPane1 = new JScrollPane(expressionEditArea);

        resultField = new JTextPane();
        resultField.setContentType("text/html");
        JScrollPane scrollPane2 = new JScrollPane(resultField);

        DefaultTableColumnModel tableColumnModel = new DefaultTableColumnModel();
        TableColumn col = new TableColumn(0);
        col.setHeaderValue("Verbindung");
        tableColumnModel.addColumn(col);

        col = new TableColumn(1);
        col.setHeaderValue("Stoffmenge [mol]");
        DefaultTableCellRenderer r = new DefaultTableCellRenderer();
        r.setHorizontalAlignment(DefaultTableCellRenderer.RIGHT);
        col.setCellRenderer(r);
        tableColumnModel.addColumn(col);

        col = new TableColumn(2);
        col.setHeaderValue("Masse [g]");
        r = new DefaultTableCellRenderer();
        r.setHorizontalAlignment(DefaultTableCellRenderer.RIGHT);
        col.setCellRenderer(r);
        tableColumnModel.addColumn(col);
        tabelle = new JTable(mengentabelle, tableColumnModel);
        JScrollPane scrollPane3 = new JScrollPane(tabelle);

        DefaultButtonModel bttnModel1 = new DefaultButtonModel();
        JButton bttn1 = new JButton();
        bttn1.setModel(bttnModel1);
        bttn1.addActionListener(this);
        bttn1.setText("Eingabe prfen");
        compileButton = bttn1;
        container.add(titel1);
        container.add(scrollPane1);
        container.add(scrollPane2);
        container.add(scrollPane3);
        container.add(bttn1);
    }

    @Override
    public void internalFrameOpened(InternalFrameEvent e) {

    }

    @Override
    public void internalFrameClosing(InternalFrameEvent e) {
        int option = JOptionPane.showConfirmDialog
                (CalculatorWindow.this,
                 "Really Close",
                 "Exit",
                 JOptionPane.YES_NO_OPTION);
        if (option == JOptionPane.YES_OPTION){
            dispose();
        }
    }

    @Override
    public void internalFrameClosed(InternalFrameEvent e) {

    }

    @Override
    public void internalFrameIconified(InternalFrameEvent e) {

    }

    @Override
    public void internalFrameDeiconified(InternalFrameEvent e) {

    }

    @Override
    public void internalFrameActivated(InternalFrameEvent e) {

    }

    @Override
    public void internalFrameDeactivated(InternalFrameEvent e) {

    }

    @Override
    public void addLayoutComponent(String name, Component comp) {

    }

    @Override
    public void removeLayoutComponent(Component comp) {

    }

    @Override
    public Dimension preferredLayoutSize(Container parent) {
        return new Dimension (200, 200);
    }

    @Override
    public Dimension minimumLayoutSize(Container parent) {
        return new Dimension(100, 100);
    }

    @Override
    public void layoutContainer(Container parent) {
        Insets insets = parent.getInsets();
        final int maxWidth = parent.getWidth() - (insets.left + insets.right);
        final int maxHeight = parent.getHeight() - (insets.top + insets.bottom);
        final int x0 = insets.left;
        final int y0 = insets.top;

        //  compute the placement bounds of the instrument display component
        Component c = parent.getComponent(0);  //  titel1
        c.setBounds(x0 + 10, y0, maxWidth/2, 34);
        c = parent.getComponent(1);  // Textfeld Eingabe Reaktionsgleichung
        c.setBounds(x0 + 10, y0 + 34, maxWidth - 20, maxHeight/4 - 44);
        c = parent.getComponent(2);  // Textfeld Ausgabe Reaktionsgleichung
        c.setBounds(x0 + 10, y0 + maxHeight/4 + 10, maxWidth - 20, maxHeight/4 - 20);
        c = parent.getComponent(3);  //  tabelle
        c.setBounds(x0 + 10, y0 + maxHeight/2 + 10, maxWidth - 20, maxHeight/2 - 20);
        c = parent.getComponent(4);  // Button

        c.setBounds(x0 + maxWidth/2, y0 + 2, maxWidth - maxWidth/2 - 10, 30);
    }

    @Override
    public void actionPerformed(ActionEvent e) {

        if (e.getSource() == compileButton) {
            this.compileExpression(e, expressionEditArea);
        }
    }

    /**
     * Verarbeitung des Inhalts des Textfeldes, in das eine Formel
     * eingegeben wird.
     * @param e
     * @param editor
     */
    private void compileExpression(ActionEvent e, JTextArea editor) {

        String stringToParse = "";

        try {
            stringToParse = document.getText(0, document.getLength());
        }
        catch (BadLocationException ex) {

        }
        FormulaParser parser = new FormulaParser(model.periodensystem());
        Reaktionsgleichung expr;
        try {
            expr = parser.parseFormula(stringToParse);
        }
        catch (NotificationOfSyntaxError ex) {
            int sp = ex.startPosition();
            int ep = ex.endPosition();
            editor.select(sp, ep);
            editor.getCaret().setSelectionVisible(true);
            JOptionPane.showMessageDialog(null,
                    ex.getErrorID(),
                    "Synax Error",
                    JOptionPane.INFORMATION_MESSAGE);
            editor.requestFocus();
            return;
        }
        //if (!expr.istWohlgeformt()) {
        AnalyseErgebnisGleichung res = expr.istWohlgeformtXX();
        if (!res.istFehlerfrei()) {

            StringBuilder sb1 = new StringBuilder(100);
            sb1.append("Die Stoffmengen dieser Reaktion sind nicht ausgeglichen:\n\n");
            res.fehlerbeschreibung(sb1);
            System.out.println(sb1.toString());
            JOptionPane.showMessageDialog(null,
                    sb1.toString(),
                    "Fehlerhafte Gleichung",
                    JOptionPane.ERROR_MESSAGE);
            return;
        }
        //  expr in html umwandeln und anzeigen
        //  Molare Massen berechnen und anzeigen
        LinkedList<Stoffmasse> pm;
        pm = expr.berechneMassen();
        mengentabelle.setRowCount(0);
        Iterator<Stoffmasse> iterator = pm.listIterator();
        while (iterator.hasNext()) {
            Stoffmasse item = iterator.next();
            String [] row = new String[3];
            StringBuilder sb1 = new StringBuilder(100);
            sb1.append("<html><p>");
            item.substance().toHtml(sb1);
            sb1.append("</p></html>");
            row[0] = sb1.toString();
            row[1] = Integer.toString(item.stoffmenge());
            row[2] = String.format("%.2f", item.stoffmasse());
            mengentabelle.addRow(row);
        }
        StringBuilder sb = new StringBuilder(1000);
        //sb.append("<p>");
        expr.toHtml(sb);
        //sb.append("</p>");
        String reaktion = sb.toString();
        resultField.setText(reaktion);
    }
}