
package window;

import java.awt.Color;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyVetoException;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

/**
 *
 * @author B. Gaertner
 */
public class MainWindow extends JFrame
         implements Runnable {

    /**
     * A reference to an instance that stores application data and implements application features.
     * @see ApplicationModel
     */
    private ApplicationModel model = null;
    JDesktopPane desktop;


    /**
     * The entry point of the program.
     */
    @Override
    public void run() {
        model = new ApplicationModel();
        this.createGUI();
    }

     /**
     * Set up the components of the main window and make the window visible.
     */
    private void createGUI() {

        try {
            UIManager.setLookAndFeel(
                UIManager.getSystemLookAndFeelClassName());
            }
        catch (UnsupportedLookAndFeelException |
               ClassNotFoundException |
               InstantiationException |
               IllegalAccessException ex) {}
        JFrame.setDefaultLookAndFeelDecorated(true);

        //  jetzt wird fr alle JFrame und alle JInternalFrame
        //  das Darstellungsthema des Betriebssystems verwendet.

        this.setTitle("Periodensystem");
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        desktop = new JDesktopPane();
        desktop.setBackground(Color.WHITE);
        this.setContentPane(desktop);
        this.createMenuBar(this);
        pack();
        GraphicsDevice gd = GraphicsEnvironment
                .getLocalGraphicsEnvironment()
                .getDefaultScreenDevice();
        int width = gd.getDisplayMode().getWidth();
        int height = gd.getDisplayMode().getHeight();
        this.setSize(width*9/10, height*9/10);
        this.setLocationRelativeTo(null); // center on screen
        this.setVisible(true);
    }

    /**
     * Eine Instanz agiert als ActionListener eines JMenuItem.
     * Sie hat die Aufgabe, das Fenster zu ffnen, das das
     * Periodensystem als Tabelle anzeigt.
     */
    private class TableCreator implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            TableWindow tw = new TableWindow(model, getSize());
            tw.setVisible(true);
            desktop.add(tw);
            desktop.moveToFront(tw);
            try {
                tw.setSelected(true);
            }
            catch (PropertyVetoException ex) {

            }
        }
    }

    /**
     * Eine Instanz agiert als ActionListener eines JMenuItem.
     * Sie hatdie Aufgabe, den Stoffmengenrechner zu ffnen.
     */
    private class CalculatorCreator implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            CalculatorWindow cw = new CalculatorWindow(model, getSize());
            cw.setVisible(true);
            desktop.add(cw);
            desktop.moveToFront(cw);
            try {
                cw.setSelected(true);
            }
            catch (PropertyVetoException ex) {

            }
        }
    }


    private class QuitHandler implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            System.exit(0);
        }
    }

    /**
     * Create the menu bar. The menu bar contains the File menu which in turn
     * contains two menu options.
     */
    private void createMenuBar(JFrame f) {
        JMenuBar menuBar = new JMenuBar();
        JMenu fileMenu = new JMenu("Periodensystem");

        JMenuItem itemOpenTable = new JMenuItem("Tabelle");
        itemOpenTable.addActionListener(new TableCreator());
        fileMenu.add(itemOpenTable);

        JMenuItem itemOpenCalculator = new JMenuItem("Rechner");
        itemOpenCalculator.addActionListener(new CalculatorCreator());
        fileMenu.add(itemOpenCalculator);

        JMenuItem itemQuit = new JMenuItem("Beenden");
        itemQuit.addActionListener(new QuitHandler());
        fileMenu.add(itemQuit);

        menuBar.add(fileMenu);
        f.setJMenuBar(menuBar);
    }
}
