
package window;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import javax.swing.ButtonGroup;
import javax.swing.JComponent;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JPopupMenu;

import notions.Periodensystem;
import notions.Element;

/**
 * Diese Klasse verwendet eine Datenstruktur, die es dem Zeichenverfahren
 * ermglicht, effizient zu bestimmen, welche Teile der Tabelle in den
 * Inhaltsbereich des Fensters fallen. Das Zeichnen von Tabellenteilen,
 * die im Fenster nicht sichtbar sind, kann damit vermieden werden, was ein
 * rasches Scrollen untersttzt.
 * @author B. Gaertner
 */
public class ScrollingTablePaintWindow extends JComponent
            implements MouseListener, // mouse button events
                       MouseMotionListener //Mouse position tracking
{

    TableWindow model;
    Periodensystem pse;
    Element [][] elementIndices;
    final int offset = 10;
    final int delta = 26;
    final int size = 24;
    boolean showLongTable;

    public ScrollingTablePaintWindow (TableWindow m) {
        super();
        showLongTable = false;
        model = m;
        pse = m.applicationModel.periodensystem();
        this.setup();
        this.setupTableLayout();
    }

    private void setupTableLayout() {
        this. recreateTable();
    }

    protected void recreateTable() {
        if (showLongTable) {
            this.createLongTableLayout();
        } else {
            this.createShortTableLayout();
        }
    }

    private void createLongTableLayout() {
        elementIndices = new Element[7][32];
        for (int i = 0; i < elementIndices.length; i++) {
            for (int j = 0; j < elementIndices[i].length; j ++) {
                elementIndices [i][j] = null;
            }
        }
        elementIndices[0][0] = pse.elementMitOrdnungszahl(1);
        elementIndices[0][31] = pse.elementMitOrdnungszahl(2);
        int c1 = 3;     // Lithium
        int c2 = 11;    // Natrium
        for (int j = 0; j < 32; j++) {
            if (j < 2) {
                elementIndices[1][j] = pse.elementMitOrdnungszahl(c1 + j);
                elementIndices[2][j] = pse.elementMitOrdnungszahl(c2 + j);
            } else if (j > 25) {
                elementIndices[1][j] = pse.elementMitOrdnungszahl(c1 + j - 24);
                elementIndices[2][j] = pse.elementMitOrdnungszahl(c2 + j - 24);
            }
        }
        c1 = 19;   //  Kalium
        c2 = 37;   //  Rubidium
        for (int j = 0; j < 32; j++) {
            if (j < 3) {
                elementIndices[3][j] = pse.elementMitOrdnungszahl(c1 + j);
                elementIndices[4][j] = pse.elementMitOrdnungszahl(c2 + j);
            } else if (j > 16) {
                elementIndices[3][j] = pse.elementMitOrdnungszahl(c1 + j - 14);
                elementIndices[4][j] = pse.elementMitOrdnungszahl(c2 + j - 14);
            }
        }
        c1 = 55;   //  Csium
        c2 = 87;   //  Franzium
        for (int j = 0; j < 32; j++) {
            elementIndices[5][j] = pse.elementMitOrdnungszahl(c1 + j);
            if (j + c2 <= pse.elementanzahl()) {
            elementIndices[6][j] = pse.elementMitOrdnungszahl(c2 + j);
            }
        }
    }

    private void createShortTableLayout() {
        elementIndices = new Element[9][18];
        for (int i = 0; i < elementIndices.length; i++) {
            for (int j = 0; j < elementIndices[i].length; j ++) {
                elementIndices [i][j] = null;
            }
        }
        elementIndices[0][0] = pse.elementMitOrdnungszahl(1);
        elementIndices[0][17] = pse.elementMitOrdnungszahl(2);
        int c1 = 3;
        int c2 = 11;
        for (int j = 0; j < 18; j++) {
            if (j < 2) {
                elementIndices[1][j] = pse.elementMitOrdnungszahl(c1 + j);
                elementIndices[2][j] = pse.elementMitOrdnungszahl(c2 + j);
            } else if (j > 11) {
                elementIndices[1][j] = pse.elementMitOrdnungszahl(c1 + j - 10);
                elementIndices[2][j] = pse.elementMitOrdnungszahl(c2 + j - 10);
            }
        }
        c1 = 19;
        c2 = 37;
        for (int j = 0; j < 18; j++) {
            elementIndices[3][j] = pse.elementMitOrdnungszahl(c1 + j);
            elementIndices[4][j] = pse.elementMitOrdnungszahl(c2 + j);
        }

        c1 = 55;
        c2 = 87;
        for (int j = 0; j < 18; j++) {
            if (j < 2) {
                elementIndices[5][j] = pse.elementMitOrdnungszahl(c1 + j);
                elementIndices[6][j] = pse.elementMitOrdnungszahl(c2 + j);
            } else if (j > 2) {
                elementIndices[5][j] = pse.elementMitOrdnungszahl(c1 + j + 14);
                if (c2 + j + 14 <= pse.elementanzahl()) {
                    elementIndices[6][j] = pse.elementMitOrdnungszahl(c2 + j + 14);
                }
            }
        }
        // Lanthanoide und Actinoide platzieren:
        c1 = 57;
        c2 = 89;
        for (int j = 0; j < 15; j++) {
            elementIndices[7][j + 2] = pse.elementMitOrdnungszahl(c1 + j);
            elementIndices[8][j + 2] = pse.elementMitOrdnungszahl(c2 + j);
        }
    }

    /**
     * Zwei Instanzen dieser Klasse werden verwendet, um zwischen langer und
     * kurzer Tabelle zu wechseln. Jede Instanz speichert dauerhaft den Zustand,
     * den sie einschalten soll.
     */
    private class Switch implements ActionListener {

        private boolean v;

        /**
         * Der Konstruktor erzeugt eine Instanz, die in den Zustand  b
         * schaltet, wenn sie die Methode  actionPerformed  ausfhrt.
         * @param b
         */
        public Switch(boolean b) {
            v = b;
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            showLongTable = v;
            //  Die drei folgenden Aufrufe gehen an die Instanz von
            //  ScrollingTablePaintWindow, die die Ausfhrungsumgebung
            //  fr die Instanz von Switch bereitstellt. Die Bezeichner
            //  this  und  super  sind hier nicht erlaubt.
            recreateTable();  //  Tabelle im neuen Format herstellen.
            revalidate();     //  scrollbars neu berechnen
            repaint();        //  Fensterinhalt neu zeichnen
        }
    }

    private void setup () {
        ButtonGroup gruppe = new ButtonGroup();
        JPopupMenu menu = new JPopupMenu();
        JRadioButtonMenuItem item1 = new JRadioButtonMenuItem("kurze Tabelle", ! showLongTable);
        JRadioButtonMenuItem item2 = new JRadioButtonMenuItem("lange Tabelle", showLongTable);

        // item1.setFont(new Font("SanSerif", Font.PLAIN, 18));
        // item2.setFont(new Font("SanSerif", Font.PLAIN, 18));
        item1.addActionListener(new Switch(false));
        item2.addActionListener(new Switch(true));
        gruppe.add(item1);
        gruppe.add(item2);
        menu.add(item1);
        menu.add(item2);
        this.setComponentPopupMenu(menu);
        this.addMouseListener(this);
        //this.addMouseMotionListener(this);
    }

    /**
     * Das bergeordnete JScrollPane bentigt diese Angabe.
     * @return
     */
    @Override
    public Dimension getPreferredSize() {
        int spalten = showLongTable ? 32 : 18;
        int zeilen  = showLongTable ? 7 : 9;
        return new Dimension (delta * spalten + 2*offset, delta*zeilen + 2*offset);
    }

    @Override
    public void paintComponent(Graphics arg) {
        //super.paintComponent(arg);
        Dimension componentSize = this.getSize();
        Insets insets = this.getInsets();

        Rectangle bereich = new Rectangle(insets.left, insets.top,
                componentSize.width - (insets.left + insets.right),
                componentSize.height - (insets.top + insets.bottom));

        Color currentColor = arg.getColor();
        arg.setColor(Color.WHITE /*this.getBackground() */);
        arg.fillRect(bereich.x, bereich.y, bereich.width, bereich.height);
        arg.setColor(currentColor);
        Graphics2D g = (Graphics2D) arg;
        Stroke st = g.getStroke();
        this.InhaltZeichnen(g, bereich);
        g.setStroke(st);
    }

    /**
     * In dieser Methode werden die Anweisungen niedergeschrieben,
     * die das Zeichnen in den Inhaltsbereich des Darstellungselements
     * bewirken.
     * @param g Instanz von Graphics2D, das Zeicheninstrument
     * @param bereich Instanz von Rectangle, die den Bereich
     * begrenzt, in den gezeichnet werden darf.
     */
    private void InhaltZeichnen(Graphics2D g, Rectangle bereich) {

        Font fnt = new Font("SanSerif", Font.PLAIN, 12);
        FontMetrics metrik = g.getFontMetrics(fnt);
        Rectangle vr = this.getVisibleRect();
        Rectangle cr = g.getClipBounds();

        /*
        System.out.println("visible rect = " + vr.toString());
        System.out.println("clip rect = " + cr.toString());
        System.out.println("----------------------");
        */

        int yInit = (cr.y - offset) / delta;
        int xInit = (cr.x - offset) / delta;
        int yEnd = (cr.y + cr.height - offset) / delta;

        if ((cr.y + cr.height - offset) % delta > 0) {
            yEnd += 1;
        }
        int xEnd = (cr.x + cr.width - offset) / delta;
        if ((cr.x + cr.width - offset) % delta > 0) {
            xEnd += 1;
        }

        for (int i = yInit; i < Math.min(yEnd, elementIndices.length); i++) {
            for (int j = xInit; j < Math.min(xEnd, elementIndices[i].length); j ++) {
                Element elem = elementIndices [i][j];
                if (elem != null) {
                    int xPos = offset + delta*j;
                    int yPos = offset + delta*i;
                    g.setColor(elem.serie().farbe());
                    g.fillRect(xPos, yPos, size, size);
                    String elementSymbol = elem.elementsymbol();
                    int breite = metrik.stringWidth(elementSymbol);
                    int deltaX = size / 2 - breite / 2;
                    int deltaY = size / 2 + metrik.getDescent();

                    g.setFont(fnt);
                    g.setColor(new Color (0, 0, 0));
                    g.drawString(elementSymbol, xPos + deltaX, yPos + deltaY);
                    if (elem == model.currentSelection) {
                        g.setColor(Color.RED);
                        g.drawRect(xPos, yPos, size - 1, size - 1);
                    }
                }
            }
        }

    }

    //  Methoden der Schnittstelle MouseListener

    @Override
    public void mouseClicked(MouseEvent e) {
        int x = e.getX();
        int y = e.getY();
        Element elem = this.positionToElement(x, y);
        if (model.currentSelection != elem) {
            //model.currentSelection = elem;
            this.repaint();  // das knnte man besser machen
            model.updateSelection(elem);
        }
    }

    @Override
    public void mousePressed(MouseEvent e) {

    }

    @Override
    public void mouseReleased(MouseEvent e) {

    }

    @Override
    public void mouseEntered(MouseEvent e) {

    }

    @Override
    public void mouseExited(MouseEvent e) {
        this.clearPosition();
    }

    //  Methoden der Schnittstelle  MouseMotionListener

    @Override
    public void mouseDragged(MouseEvent e) {
        this.setPosition(e.getX(), e.getY());
    }

    @Override
    public void mouseMoved(MouseEvent e) {
        this.setPosition(e.getX(), e.getY());
    }

    //  private Methoden fr die Positionsauswertung

    private void clearPosition() {

    }

    private void setPosition(int x, int y) {
        Element currentPos;
        currentPos = this.positionToElement(x, y);

    }

    /**
     * Die Umwandlung der Mausposition in einen Elementindex verlangt eine Idee.
     * Wichtig ist, dass diese Umwandlung sehr schnell geschehen kann, da
     * Mausbewegungen keine langen Bearbeitungszeiten zulassen. Hier wird der
     * Darstellungsbereich in Gitterpltze eingeteilt. Ein zweidimensionales
     * Feld wird verwendet, um Gitterpltze auf die Ordnungszahlen der Elemente
     * abzubilden. Dieses Verfahren verlangt Speicherplatz fr das zweidimensionale
     * Feld und einmaligen Rechenaufwand fr die Speicherung der Ordnungszahlen in
     * dieses Feld. Dafr geschieht die Bestimmung der Ordnungszahl des Elements
     * auf das der Mauszeiger verweist mit nur zwei Umrechnungen und einem
     * Feldzugriff. Man knnte in dem Feld auch sogleich die Elemente speichern,
     * das wrde einen weiteren Zugriffsschritt vermeiden.
     * @param x
     * @param y
     * @return
     */
    private Element positionToElement(int x, int y) {

        int xt = (x - offset) / delta;
        int yt = (y - offset) / delta;

        Element markiertesElement = null;
        try {
            markiertesElement = elementIndices[yt][xt];
        }
        catch (ArrayIndexOutOfBoundsException ex) {
            //  keine Aktion erforderlich, es handelt sich
            //  um eine erwartete Ausnahme.
        }

        return markiertesElement;
    }
}