
package window;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import notions.Periodensystem;
import notions.Element;

/**
 * Diese Klasse verwendet eine Datenstruktur, die es dem Zeichenverfahren
 * ermglicht, effizient zu bestimmen, welche Teile der Tabelle in den
 * Inhaltsbereich des Fensters fallen. Das Zeichnen von Tabellenteilen,
 * die im Fenster nicht sichtbar sind, kann damit vermieden werden, was ein
 * rasches Scrollen untersttzt.
 * @author B. Gaertner
 */
public class ScrollingTablePaintWindowOLD extends JComponent
            implements MouseListener, // mouse button events
                       MouseMotionListener //Mouse position tracking
{

    TableWindow model;
    Periodensystem pse;
    Element [][] elementIndices;
    final int offset = 10;
    final int delta = 26;
    final int size = 24;
    boolean showLongTable;

    public ScrollingTablePaintWindowOLD (TableWindow m) {
        super();
        model = m;
        pse = m.applicationModel.periodensystem();
        this.setup();
    }

    private class SwitchToShort implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            showLongTable = false;
        }

    }

    private class SwitchToLong implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            showLongTable = true;

        }

    }

    private void setup () {
        JPopupMenu menu = new JPopupMenu();
        JMenuItem item1 = new JMenuItem("kurze Tabelle");
        JMenuItem item2 = new JMenuItem("lange Tabelle");
        item1.addActionListener(new SwitchToShort());
        item2.addActionListener(new SwitchToLong());
        menu.add(item1);
        menu.add(item2);
        this.setComponentPopupMenu(menu);
        this.addMouseListener(this);
        this.addMouseMotionListener(this);
        elementIndices = new Element[7][32];
        for (int i = 0; i < elementIndices.length; i++) {
            for (int j = 0; j < elementIndices[i].length; j ++) {
                elementIndices [i][j] = null;
            }
        }
        elementIndices[0][0] = pse.elementMitOrdnungszahl(1);
        elementIndices[0][31] = pse.elementMitOrdnungszahl(2);
        int c1 = 2;
        int c2 = 10;
        for (int j = 0; j < 32; j++) {
            if (j < 2) {
                elementIndices[1][j] = pse.elementMitOrdnungszahl(c1 + j + 1);
                elementIndices[2][j] = pse.elementMitOrdnungszahl(c2 + j + 1);
            } else if (j > 25) {

                elementIndices[1][j] = pse.elementMitOrdnungszahl(c1 + j - 23);
                elementIndices[2][j] = pse.elementMitOrdnungszahl(c2 + j - 23);
            }
        }
        c1 = 18;
        c2 = 36;
        for (int j = 0; j < 32; j++) {
            if (j < 3) {
                elementIndices[3][j] = pse.elementMitOrdnungszahl(c1 + j + 1);
                elementIndices[4][j] = pse.elementMitOrdnungszahl(c2 + j + 1);
            } else if (j > 16) {

                elementIndices[3][j] = pse.elementMitOrdnungszahl(c1 + j - 13);
                elementIndices[4][j] = pse.elementMitOrdnungszahl(c2 + j - 13);
            }
        }
        c1 = 55;
        c2 = 87;
        for (int j = 0; j < 32; j++) {
            elementIndices[5][j] = pse.elementMitOrdnungszahl(c1 + j);
            if (j + c2 <= pse.elementanzahl()) {
            elementIndices[6][j] = pse.elementMitOrdnungszahl(c2 + j);
            }
        }
    }

    /**
     * Das bergeordnete JScrollPane bentigt diese Angabe.
     * @return
     */
    @Override
    public Dimension getPreferredSize() {
        return new Dimension (delta * 32 + 2*offset, delta*7 + 2*offset);
    }

    @Override
    public void paintComponent(Graphics arg) {
        //super.paintComponent(arg);
        Dimension componentSize = this.getSize();
        Insets insets = this.getInsets();

        Rectangle bereich = new Rectangle(insets.left, insets.top,
                componentSize.width - (insets.left + insets.right),
                componentSize.height - (insets.top + insets.bottom));

        Color currentColor = arg.getColor();
        arg.setColor(this.getBackground());
        arg.fillRect(bereich.x, bereich.y, bereich.width, bereich.height);
        arg.setColor(currentColor);
        Graphics2D g = (Graphics2D) arg;
        Stroke st = g.getStroke();
        this.InhaltZeichnen(g, bereich);
        g.setStroke(st);
    }

    /**
     * In dieser Methode werden die Anweisungen niedergeschrieben,
     * die das Zeichnen in den Inhaltsbereich des Darstellungselements
     * bewirken.
     * @param g Instanz von Graphics2D, das Zeicheninstrument
     * @param bereich Instanz von Rectangle, die den Bereich
     * begrenzt, in den gezeichnet werden darf.
     */
    private void InhaltZeichnen(Graphics2D g, Rectangle bereich) {

        Font fnt = new Font("SanSerif", Font.PLAIN, 12);
        FontMetrics metrik = g.getFontMetrics(fnt);
        Rectangle vr = this.getVisibleRect();
        Rectangle cr = g.getClipBounds();
        /*
        System.out.println("visible rect = " + vr.toString());
        System.out.println("clip rect = " + cr.toString());
        System.out.println("----------------------");
        */
        int yInit = (cr.y - offset) / delta;
        int xInit = (cr.x - offset) / delta;
        int yEnd = (cr.y + cr.height - offset) / delta;

        if ((cr.y + cr.height - offset) % delta > 0) {
            yEnd += 1;
        }
        int xEnd = (cr.x + cr.width - offset) / delta;
        if ((cr.x + cr.width - offset) % delta > 0) {
            xEnd += 1;
        }

        for (int i = yInit; i < Math.min(yEnd, elementIndices.length); i++) {
            for (int j = xInit; j < Math.min(xEnd, elementIndices[i].length); j ++) {
                Element elem = elementIndices [i][j];
                if (elem != null) {
                    int xPos = offset + delta*j;
                    int yPos = offset + delta*i;
                    g.setColor(elem.serie().farbe());
                    g.fillRect(xPos, yPos, size, size);
                    String elementSymbol = elem.elementsymbol();
                    int breite = metrik.stringWidth(elementSymbol);
                    int deltaX = size / 2 - breite / 2;
                    int deltaY = size / 2 + metrik.getDescent();

                    g.setFont(fnt);
                    g.setColor(new Color (0, 0, 0));
                    g.drawString(elementSymbol, xPos + deltaX, yPos + deltaY);
                    if (elem == model.currentSelection) {
                        g.setColor(Color.RED);
                        g.drawRect(xPos, yPos, size - 1, size - 1);
                    }
                }
            }
        }

    }

    //  Methoden der Schnittstelle MouseListener

    @Override
    public void mouseClicked(MouseEvent e) {
        int x = e.getX();
        int y = e.getY();
        Element elem = this.positionToElement(x, y);
        if (model.currentSelection != elem) {
            //model.currentSelection = elem;
            this.repaint();  // das knnte man besser machen
            model.updateSelection(elem);
        }
    }

    @Override
    public void mousePressed(MouseEvent e) {

    }

    @Override
    public void mouseReleased(MouseEvent e) {

    }

    @Override
    public void mouseEntered(MouseEvent e) {

    }

    @Override
    public void mouseExited(MouseEvent e) {
        this.clearPosition();
    }

    //  Methoden der Schnittstelle  MouseMotionListener

    @Override
    public void mouseDragged(MouseEvent e) {
        this.setPosition(e.getX(), e.getY());
    }

    @Override
    public void mouseMoved(MouseEvent e) {
        this.setPosition(e.getX(), e.getY());
    }

    //  private Methoden fr die Positionsauswertung

    private void clearPosition() {

    }

    private void setPosition(int x, int y) {
        Element currentPos;
        currentPos = this.positionToElement(x, y);

    }

    /**
     * Die Umwandlung der Mausposition in einen Elementindex verlangt eine Idee.
     * Wichtig ist, dass diese Umwandlung sehr schnell geschehen kann, da
     * Mausbewegungen keine langen Bearbeitungszeiten zulassen. Hier wird der
     * Darstellungsbereich in Gitterpltze eingeteilt. Ein zweidimensionales
     * Feld wird verwendet, um Gitterpltze auf die Ordnungszahlen der Elemente
     * abzubilden. Dieses Verfahren verlangt Speicherplatz fr das zweidimensionale
     * Feld und einmaligen Rechenaufwand fr die Speicherung der Ordnungszahlen in
     * dieses Feld. Dafr geschieht die Bestimmung der Ordnungszahl des Elements
     * auf das der Mauszeiger verweist mit nur zwei Umrechnungen und einem
     * Feldzugriff. Man knnte in dem Feld auch sogleich die Elemente speichern,
     * das wrde einen weiteren Zugriffsschritt vermeiden.
     * @param x
     * @param y
     * @return
     */
    private Element positionToElement(int x, int y) {

        int xt = (x - offset) / delta;
        int yt = (y - offset) / delta;

        Element elementIndex = null;
        try {
            elementIndex = elementIndices[yt][xt];
        }
        catch (ArrayIndexOutOfBoundsException ex) {
            //  keine Aktion erforderlich, es handelt sich
            //  um eine erwartete Ausnahme.
        }

        return elementIndex;
    }
}

