
package window;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import notions.Periodensystem;
import notions.Element;

/**
 *
 * @author B. Gaertner
 */
public class TablePaintWindow extends JComponent
            implements MouseListener, // mouse button events
                       MouseMotionListener //Mouse position tracking
{

    Periodensystem pse;
    int [][] elementIndices;

    public TablePaintWindow (Periodensystem p) {
        super();
        pse = p;
        this.setup();
    }

    private void setup () {

        JPopupMenu menu = new JPopupMenu();
        JMenuItem item1 = new JMenuItem("kurze Tabelle");
        JMenuItem item2 = new JMenuItem("lange Tabelle");
        item1.addActionListener(null);
        item2.addActionListener(null);
        menu.add(item1);
        menu.add(item2);
        this.setComponentPopupMenu(menu);
        this.addMouseListener(this);
        this.addMouseMotionListener(this);
        elementIndices = new int[7][32];
        for (int i = 0; i < elementIndices.length; i++) {
            for (int j = 0; j < elementIndices[i].length; j ++) {
                elementIndices [i][j] = -1;
            }
        }
        elementIndices[0][0] = 1;
        elementIndices[0][31] = 2;
        int c1 = 2;
        int c2 = 10;
        for (int j = 0; j < 32; j++) {
            if (j < 2) {
                elementIndices[1][j] = c1 + j + 1;
                elementIndices[2][j] = c2 + j + 1;
            } else if (j > 25) {

                elementIndices[1][j] = c1 + j - 23;
                elementIndices[2][j] = c2 + j - 23;
            }
        }
        c1 = 18;
        c2 = 36;
        for (int j = 0; j < 32; j++) {
            if (j < 3) {
                elementIndices[3][j] = c1 + j + 1;
                elementIndices[4][j] = c2 + j + 1;
            } else if (j > 16) {

                elementIndices[3][j] = c1 + j - 13;
                elementIndices[4][j] = c2 + j - 13;
            }
        }
        c1 = 55;
        c2 = 87;
        for (int j = 0; j < 32; j++) {
            elementIndices[5][j] = c1 + j;
            if (j + c2 <= 112) {
            elementIndices[6][j] = c2 + j;
            }
        }

        //this.setOpaque(true);
    }

    @Override
    public Dimension getPreferredSize() {
        return new Dimension (700, 300);
    }

    @Override
    public void paintComponent(Graphics arg) {
        super.paintComponent(arg);
        Dimension size = this.getSize();
        Insets insets = this.getInsets();

        Rectangle bereich = new Rectangle(insets.left, insets.top,
                size.width - (insets.left + insets.right),
                size.height - (insets.top + insets.bottom));

        Color currentColor = arg.getColor();
        arg.setColor(this.getBackground());
        arg.fillRect(bereich.x, bereich.y, bereich.width, bereich.height);
        arg.setColor(currentColor);
        Graphics2D g = (Graphics2D) arg;
        Stroke st = g.getStroke();
        this.InhaltZeichnen(g, bereich);
        g.setStroke(st);
    }

    /**
     * In dieser Methode werden die Anweisungen niedergeschrieben,
     * die das Zeichnen in den Inhaltsbereich des Darstellungselements
     * bewirken.
     * @param g Instanz von Graphics2D, das Zeicheninstrument
     * @param bereich Instanz von Rectangle, die den Bereich
     * begrenzt, in den gezeichnet werden darf.
     */
    private void InhaltZeichnen(Graphics2D g, Rectangle bereich) {
        Rectangle vr = this.getVisibleRect();
        System.out.println("visible rect = " + vr.toString());
        for (int i = 0; i < pse.elementanzahl(); i++) {
            Element element = pse.elementMitOrdnungszahl(i);
            int p = element.periode();
            int gr = element.gruppennummer();
            int y = p - 1;
            int x = gr - 1 + (gr == 3 && p >= 6 ? element.offsetFromFirstGroup() - 3 : 0)
                           + (gr > 3 ? 14 : 0);
            g.setColor(element.serie().farbe());
            g.fillRect(10 + 20*x, 10 + 20*y, 18, 18);
        }
    }

    //  Methoden der Schnittstelle MouseListener

    @Override
    public void mouseClicked(MouseEvent e) {
        int x = e.getX();
        int y = e.getY();
    }

    @Override
    public void mousePressed(MouseEvent e) {

    }

    @Override
    public void mouseReleased(MouseEvent e) {

    }

    @Override
    public void mouseEntered(MouseEvent e) {

    }

    @Override
    public void mouseExited(MouseEvent e) {
        this.clearPosition();
    }

    //  Methoden der Schnittstelle  MouseMotionListener

    @Override
    public void mouseDragged(MouseEvent e) {
        this.setPosition(e.getX(), e.getY());
    }

    @Override
    public void mouseMoved(MouseEvent e) {
        this.setPosition(e.getX(), e.getY());
    }

    //  private MEthoden fr die Positionsauswertung

    private void clearPosition() {

    }

    private void setPosition(int x, int y) {
        int currentPos;
        currentPos = this.positionToElement(x, y);

    }

    /**
     * Die Umwandlung der Mausposition in einen Elementindex verlangt eine Idee.
     * Wichtig ist, dass diese Umwandlung sehr schnell geschehen kann, da
     * Mausbewegungen keine langen Bearbeitungszeiten zulassen. Hier wird der
     * Darstellungsbereich in Gitterpltze eingeteilt. Ein zweidimensionales
     * Feld wird verwendet, um Gitterpltze auf die Ordnungszahlen der Elemente
     * abzubilden. Dieses Verfahren verlangt Speicherplatz fr das zweidimensionale
     * Feld und einmaligen Rechenaufwand fr die Speicherung der Ordnungszahlen in
     * dieses Feld. Dafr geschieht die Bestimmung der Ordnungszahl des Elements
     * auf das der Mauszeiger verweist mit nur zwei Umrechnungen und einem
     * Feldzugriff. Man knnte in dem Feld auch sogleich die Elemente speichern,
     * das wrde einen weiteren Zugriffsschritt vermeiden.
     * @param x
     * @param y
     * @return
     */
    private int positionToElement(int x, int y) {

        int xt = (x - 10) / 20;
        int yt = (y - 10) / 20;

        int elementIndex = -1;
        try {
            elementIndex = elementIndices[yt][xt];
        }
        catch (ArrayIndexOutOfBoundsException ex) {

        }
        //System.out.println("ElementIndex = " + elementIndex );
        return elementIndex;
    }
}

