
package window;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;

import documenttools.StyledDocumentCreator;
import notions.Element;

/**
 *
 * @author B. Gaertner
 */
public class TableWindow extends JInternalFrame
       implements InternalFrameListener, LayoutManager {

    ApplicationModel applicationModel;
    Element currentSelection;
    JTextPane eb;
    StyleContext sc;

    public TableWindow(ApplicationModel m, Dimension d) {
        super("Tabelle",
              true, //resizable
              true, //closable
              true, //maximizable
              true);//iconifiable
        applicationModel = m;
        this.setupDocumentStyles();
        this.createGUI(d);
    }

    private void setupDocumentStyles() {
        sc = new StyleContext();
        Style st1 = sc.addStyle("BasicStyle", null);  // no parent
        //st1.addAttribute(StyleConstants.FontFamily, "sanserif");
        //st1.addAttribute(StyleConstants.FontSize, 14);
        //st1.addAttribute(StyleConstants.Bold, false);
        StyleConstants.setFontSize(st1, 14);
        StyleConstants.setFontFamily(st1, "sanserif");
        StyleConstants.setBold(st1, false);


        Style st2 = sc.addStyle("Title", st1);  // derived from st1
        st2.addAttribute(StyleConstants.Bold, true);
        st2.addAttribute(StyleConstants.Underline, false);

        Style st3 = sc.addStyle("Header2", st1);
        st3.addAttribute(StyleConstants.Underline, false);

        //  The use of this style is unclear
        Style st4 = sc.addStyle("IndentedPlainText", st1);
        StyleConstants.setFirstLineIndent(st4, 120.0f);
        StyleConstants.setSpaceAbove(st4, 12.0f);
    }

    private void createGUI(Dimension d) {
        setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
           //  Fenster wird in  internalFrameClosing() mit dispose()
           //  geschlossen.
        this.addInternalFrameListener(this);
        addComponentsToPane(this.getContentPane());
        setSize(d.width*7/10, d.height*7/10);
        setLocation(10, 0);
    }

    private void addComponentsToPane(Container container) {
        container.setLayout(this);
        ScrollingTablePaintWindow tpw = new ScrollingTablePaintWindow(this);
        JScrollPane scrollPane = new JScrollPane(tpw);
        eb = new JTextPane(this.createDescription());
        eb.setEditable(false);
        container.add(scrollPane);
        container.add(eb);
    }


    public void updateSelection(Element elem) {
        currentSelection = elem;
        DefaultStyledDocument d = this.createDescription();
        eb.setDocument(d);
    }

    private DefaultStyledDocument createDescription() {
        DefaultStyledDocument d = new DefaultStyledDocument(sc);
        if (currentSelection != null) {
            StyledDocumentCreator dc = new StyledDocumentCreator(d);
            dc.setStyle("Title");
            dc.append(Integer.toString(currentSelection.ordnungszahl()));
            dc.append("  ");
            dc.append(currentSelection.elementsymbol());
            dc.nl(1);
            dc.setStyle("Header2");
            dc.append(currentSelection.elementName());
            dc.nl(1);
            dc.setStyle("IndentedPlainText");
            dc.append("Gruppe: ");
            dc.append(currentSelection.gruppe().getGroupDescription());
            dc.nl();
            dc.append("Serie: ");
            dc.append(currentSelection.serie().name());
            dc.nl();
            dc.append("Atommasse = ");
            dc.append(Float.toString(currentSelection.molekularmasse()));
            //String str = String.format("%.2f", currentSelection.molekularmasse());
            //dc.append(String.format("%.2f", currentSelection.molekularmasse()));
        }
        return d;
    }

    @Override
    public void internalFrameOpened(InternalFrameEvent e) {

    }

    @Override
    public void internalFrameClosing(InternalFrameEvent e) {
        int option = JOptionPane.showConfirmDialog
                (TableWindow.this,
                 "Really Close",
                 "Exit",
                 JOptionPane.YES_NO_OPTION);
        if (option == JOptionPane.YES_OPTION){
            eb.setDocument(new DefaultStyledDocument());
            dispose();
        }
    }

    @Override
    public void internalFrameClosed(InternalFrameEvent e) {

    }

    @Override
    public void internalFrameIconified(InternalFrameEvent e) {

    }

    @Override
    public void internalFrameDeiconified(InternalFrameEvent e) {

    }

    @Override
    public void internalFrameActivated(InternalFrameEvent e) {

    }

    @Override
    public void internalFrameDeactivated(InternalFrameEvent e) {

    }

    @Override
    public void addLayoutComponent(String name, Component comp) {

    }

    @Override
    public void removeLayoutComponent(Component comp) {

    }

    @Override
    public Dimension preferredLayoutSize(Container parent) {
        return new Dimension (200, 200);
    }

    @Override
    public Dimension minimumLayoutSize(Container parent) {
        return new Dimension(200, 100);
    }

    @Override
    public void layoutContainer(Container parent) {
        Insets insets = parent.getInsets();
        final int maxWidth = parent.getWidth() - (insets.left + insets.right);
        final int maxHeight = parent.getHeight() - (insets.top + insets.bottom);
        final int x0 = insets.left;
        final int y0 = insets.top;

        //  compute the placement bounds of the instrument display component
        Component c = parent.getComponent(0);    //  Tabelle
        c.setBounds(x0, y0 , maxWidth - 250, maxHeight);
        c = parent.getComponent(1);  //  ElementBeschreibung
        c.setBounds(x0 + maxWidth - 250, y0 , 250, maxHeight);
    }
}
